/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright© 2020 MikroElektronika d.o.o.
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be 
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE 
 * OR OTHER DEALINGS IN THE SOFTWARE. 
 */

/*!
 * \file
 *
 * \brief This file contains API for POWERBANK 2 Click driver.
 *
 * \addtogroup powerbank2 POWERBANK 2 Click Driver
 * @{
 */
// ----------------------------------------------------------------------------

#ifndef POWERBANK2_H
#define POWERBANK2_H

/**
 * Any initialization code needed for MCU to function properly.
 * Do not remove this line or clock might not be set correctly.
 */
#ifdef PREINIT_SUPPORTED
#include "preinit.h"
#endif

#ifdef MikroCCoreVersion
    #if MikroCCoreVersion >= 1
        #include "delays.h"
    #endif
#endif

#include "drv_digital_out.h"
#include "drv_digital_in.h"
#include "drv_i2c_master.h"


// -------------------------------------------------------------- PUBLIC MACROS 
/**
 * \defgroup macros Macros
 * \{
 */

/**
 * \defgroup map_mikrobus MikroBUS
 * \{
 */
#define POWERBANK2_MAP_MIKROBUS( cfg, mikrobus ) \
  cfg.scl  = MIKROBUS( mikrobus, MIKROBUS_SCL ); \
  cfg.sda  = MIKROBUS( mikrobus, MIKROBUS_SDA )
/** \} */

/**
 * \defgroup error_code Error Code
 * \{
 */
#define POWERBANK2_RETVAL  uint8_t

#define POWERBANK2_OK           0x00
#define POWERBANK2_INIT_ERROR   0xFF
/** \} */

/**
 * \defgroup Device's I2C slave address
 * \{
 */
#define POWERBANK2_SLAVE_ADR     0x4D
/** \} */

/**
 * \defgroup ADC resolution and usefull data
 * \{
 */
#define POWERBANK2_RESOLUTION   4096
#define POWERBANK2_12_BIT_DATA  0x0FFF
/** \} */

/** \} */ // End group macro 
// --------------------------------------------------------------- PUBLIC TYPES
/**
 * \defgroup type Types
 * \{
 */

/**
 * @brief Click ctx object definition.
 */
typedef struct
{
    // Modules 

    i2c_master_t i2c;

    // ctx variable 

    uint8_t slave_address;

} powerbank2_t;

/**
 * @brief Click configuration structure definition.
 */
typedef struct
{
    // Communication gpio pins 

    pin_name_t scl;
    pin_name_t sda;


    // static variable 

    uint32_t i2c_speed;
    uint8_t i2c_address;

} powerbank2_cfg_t;

/** \} */ // End types group
// ----------------------------------------------- PUBLIC FUNCTION DECLARATIONS

/**
 * \defgroup public_function Public function
 * \{
 */
 
#ifdef __cplusplus
extern "C"{
#endif

/**
 * @brief Config Object Initialization function.
 *
 * @param cfg  Click configuration structure.
 *
 * @description This function initializes click configuration structure to init state.
 * @note All used pins will be set to unconnected state.
 */
void powerbank2_cfg_setup ( powerbank2_cfg_t *cfg );

/**
 * @brief Initialization function.
 * @param powerbank2 Click object.
 * @param cfg Click configuration structure.
 * 
 * @description This function initializes all necessary pins and peripherals used for this click.
 */
POWERBANK2_RETVAL powerbank2_init ( powerbank2_t *ctx, powerbank2_cfg_t *cfg );

/**
 * @brief Generic write function.
 *
 * @param ctx          Click object.
 * @param reg          Register address.
 * @param data_buf     Data buf to be written.
 * @param len          Number of the bytes in data buf.
 *
 * @description This function writes data to the desired register.
 */
void powerbank2_generic_write ( powerbank2_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len );

/**
 * @brief Read function.
 *
 * 
 * @param ctx          Click object.
 * @param result       Result.
 *
 * @description This function reads data from the desired register.
 */
void powerbank2_read ( powerbank2_t *ctx, uint16_t *result );

/**
 * @brief Read voltage function.
 *
 * @param ctx          Click object.
 * @param v_ref  16-bit value representing power supply voltage.
 *
 * @returns result 12-bit value that represents voltage in millivolts.
 *
 * @description This function is used to calculate voltage of the connected battery.
**/
uint16_t powerbank2_read_voltage ( powerbank2_t *ctx, uint16_t v_ref );

#ifdef __cplusplus
}
#endif
#endif  // _POWERBANK2_H_

/** \} */ // End public_function group
/// \}    // End click Driver group  
/*! @} */
// ------------------------------------------------------------------------- END
